<?php

/*
 * Evolution Nova
 *
 * @author       Bahram Davodi  <dev@nova-cms.com>
 * @copyright    Copyright (C) 2008-2010 All rights reserved.
 * @license      GNU/GPLv2 http://www.gnu.org/licenses/
 * @link         http://www.nova-cms.com
 *
 */

if (!defined('FUNC_FILE')) die('Access is limited');

class file_download {
    /**
    * The ID of the possibility of resuming a file
    * 
    * @var bool
    */
    var $range;
    
    /**
    * File options
    * 
    * @var array
    */
    var $file = array(
        'size'   => '',
        'name'   => '',
        'type'   => '',
        'resume' => '',
        'speed'  => ''
    );
    
    /**
    * The function of determining the parameters for downloading files
    * 
    * @param string $file
    * @param int $resume
    * @param int $speed
    * @return void
    */
    function file_download($file, $resume=0, $speed=0, $name=''){
        if(nc_is_dir($file)) nc_http_ereor_logs("403"); elseif (!nc_is_file($file)) nc_http_ereor_logs("404");
        $fileinfo = pathinfo($file);
        $this->file = array( 
            'ptch'   => $file,
            'size'   => nc_filesize($file),
            'name'   => strstr(get_user_agent(), 'MSIE') ? preg_replace('/\./', '%2e', (!empty($name)?$name:$fileinfo['basename']), nc_substr_count((!empty($name)?$name:$fileinfo['basename']), '.') - 1) : (!empty($name)?$name:$fileinfo['basename']),
            'type'   => isset($fileinfo['extension']) ? nc_strtolower($fileinfo['extension']) : "",
            'resume' => $resume,
            'speed'  => $speed
        );     
        if($this->file['resume']){
            //If allowed to resume
            if(isset($_SERVER['HTTP_RANGE'])) $this->range = str_replace("-", "", str_replace("bytes=", "", $_SERVER['HTTP_RANGE']));
            else $this->range = 0;
        } else $this->range = 0;
    }
    
    /**
    * Function of downloading files
    * 
    * @return bool
    */
    function download(){
    global $MIME;        
        //If allowed to resume
        if($this->range) header($_SERVER['SERVER_PROTOCOL']." 206 Partial Content");
        else header($_SERVER['SERVER_PROTOCOL']." 200 OK");
        header("Pragma: public");
        header("Expires: 0");
        header("Cache-Control:");
        header("Cache-Control: public");
        header("Content-Description: File Transfer");
        header("Content-Type: ".(isset($MIME[$this->file['type']]) ? $MIME[$this->file['type']] : 'application/force-download'));
        header('Content-Disposition: attachment; filename="'.$this->file['name'].'";');
        header("Content-Transfer-Encoding: binary");        
        if($this->file['resume']) header("Accept-Ranges: bytes");
        if($this->range){
            //If allowed to resume
            header("Content-Range: bytes {$this->range}-".($this->file['size']-1)."/".$this->file['size']);
            header("Content-Length: ".($this->file['size']-$this->range));
        } else header("Content-Length: ".$this->file['size']);
        if(($speed = $this->file['speed']) > 0) $sleep_time = (8 / $speed) * 1e6;
        else $sleep_time = 0;
        /*[X]*/
        if(!SAFE_MODE AND function_exists('set_time_limit')) set_time_limit(0);
        else @ini_set('max_execution_time', 0);
        //Open the file for reading
        $handle = fopen($this->file['ptch'], 'rb');
        fseek($handle,$this->range);
        if($handle === false) return false;
        //Read the file with a given speed download
        if(@ob_get_length()>0) @ob_end_clean();
        while(!feof($handle)){
            print(fread($handle, 1024*8));
            if(function_exists('ob_flush') AND function_exists('flush') AND @ob_get_length()>0){
                ob_flush();
                flush();
            }
            usleep($sleep_time);
        } 
        fclose($handle);
        return true;
    }
}
?>