<?php

/*
 * Evolution Nova
 *
 * @author       Bahram Davodi  <dev@nova-cms.com>
 * @copyright    Copyright (C) 2008-2010 All rights reserved.
 * @license      GNU/GPLv2 http://www.gnu.org/licenses/
 * @link         http://www.nova-cms.com
 *
 */

if (!defined("FUNC_FILE")) die("Access is limited");
define("DB", true);

class sql_db{
    var $db_connect_id;
    var $query_result;
    var $num_queries = 0;
    var $total_time_db = 0;
    var $time_query = "";
    var $report_error = true;
    var $error = "";
    var $errno = "";
    var $dsn = "mysql";

    /**
    * The class constructor
    * 
    * @param array $cfg
    * @param boot $persistency
    * @return resourse
    */
    function sql_db($cfg, $persistency = true) {
        try {
            $this->db_connect_id = new PDO("{$this->dsn}:host={$cfg['host']};dbname={$cfg['name']}", $cfg['user'], $cfg['password']);
            if(!empty($cfg['charset']) AND $cfg['type']=='mysql') $this->db_connect_id->query("SET NAMES '{$cfg['charset']}'");
            return $this->db_connect_id;
        } catch (PDOException $e) {
            die("<center>Connection failed: ".$e->getMessage()."</center>");
        }
    }

    /**
    * Closes the connection to the server
    *     
    * @return bool
    */
    function sql_close() {
        if($this->db_connect_id) {
            $this->db_connect_id = null;
            return true;
        } else return false;
    }

    /**
    * Sends a request to an active database server
    *     
    * @param bool $transaction
    * @param string $query        
    * @return resourse
    **/
    function sql_query($query = "", $transaction = false) {
        if($this->db_connect_id) {
            unset($this->query_result);
            if($query != "") {
                $st = microtime(1);            
                $this->query_result = $this->db_connect_id->query($query);
                $total_tdb = round(microtime(1)-$st, 5);
                $this->total_time_db += $total_tdb;
                $this->time_query .= ($total_tdb > 0.01) ? "<div align='left' class='debbug_query'><font color='red' style='float: left;'><b>{$total_tdb}</b></font><pre>[".htmlspecialchars(trim($query), ENT_QUOTES)."]</pre></div>\n" : "<div align='left'><font color='green' style='float: left;'><b>{$total_tdb}</b></font><pre>[".htmlspecialchars(trim($query), ENT_QUOTES)."]</pre></div>\n";
            }                                                                                
            if($this->query_result) {                   
                $this->num_queries+=1;
                $this->error = $this->errno = "";
                return $this->query_result;
            } else {    
                $this->error = $this->sql_error($this->db_connect_id);
                $this->errno = $this->sql_errno($this->db_connect_id);
                if($this->report_error) nc_sql_erorr_logs($this->errno, $this->error, $query);
                return ($transaction == "END_TRANSACTION") ? true : false;
            }
        } else return false;
    }        
    
    /**
    * Returns the number of rows of query result
    * 
    * @param resourse $query_id
    * @return int
    */
    function sql_numrows($query_id = 0) {
        if(!$query_id) $query_id = $this->query_result;        
        if($query_id) return $query_id->rowCount();
        else return false;
    }

    /**
    * Returns the number of rows affected by the last INSERT, UPDATE, DELETE request to the server
    * 
    * @param resourse $query_id
    * @return int
    */
    function sql_affectedrows(){
        if($this->query_result) return $this->query_result->rowCount();
        else return false;
        
    }

    /**
    * Returns the number of fields in result zaproosa
    * 
    * @param resourse $query_id
    * @return int
    */
    function sql_numfields($query_id = 0) {
        if(!$query_id) $query_id = $this->query_result;
        if($query_id) return count($query_id->fetch(PDO::FETCH_ASSOC));
        else return false;
    }

    /**
    * Returns the name of the column at the specified index
    *                                                                                           
    * @param int $offset
    * @param resourse $query_id
    * @return string
    */
    function sql_fieldname($offset, $query_id = 0) {
        if(!$query_id) $query_id = $this->query_result;
        $row = $query_id->fetch(PDO::FETCH_ASSOC);
        $i = 0;
        if(count($row)-1>=$offset){
            foreach($row as $key => $value){
                if($i==$offset) return $key;
                $i++;
            }
        } else return false;
    }
    
    /**
    * Returns an array of processed next query result
    * 
    * @param resourse $query_id
    * @param bool $debug
    * @return array
    */
    function sql_fetchrow($query_id = 0) {
        if(!$query_id) $query_id = $this->query_result;
        if($query_id) return $query_id->fetch(PDO::FETCH_BOTH);
        else return false;
    }

    /**
    * Fetch a result row and returns an associative array, a numeric array, or both
    *     
    * @param resourse $query_id
    * @return array
    */    
    function sql_fetchrowset($query_id = 0) {
        if(!$query_id) $query_id = $this->query_result;
        if($query_id) return $query_id->fetch(PDO::FETCH_BOTH);
        else return false;
    }

    /**
    * Returns the ID, the generated AUTO_INCREMENT column with the last INSERT query to the server
    *     
    * @param resourse $query_id
    * @return int
    */
    function sql_nextid($query_id = "") {
        if($this->db_connect_id) return $this->db_connect_id->lastInsertId();
        else return false;
    }

    /**
    * Will release all memory occupied by the result, referred to by the passed function pointer
    * 
    * @param resourse $query_id
    * @return bool
    */
    function sql_freeresult($query_id = 0){
        if($query_id) {
            $query_id->closeCursor();
            $query_id = null;
            return true;
        } else return false;
    }
    
    /**
    * Returns the numeric error code from the last MySQL operation
    * 
    * @param resource $query_id
    * @return int
    */
    function sql_errno($query_id = 0){
        if(!$query_id) $query_id = $this->query_result;
        if($query_id) return $query_id->errorCode();
        else return false;
    }

    /**
    * Returns the error string of the last operation with MySQL.
    * 
    * @param resource $query_id
    * @return string
    */
    function sql_error($query_id = 0){
        if(!$query_id) $query_id = $this->query_result;
        $info = $query_id->errorInfo();
        if($query_id) return $info[2];
        else return false;
    }
}
?>
