<?php

/*
 * Evolution Nova
 *
 * @author       Bahram Davodi  <dev@nova-cms.com>
 * @copyright    Copyright (C) 2008-2010 All rights reserved.
 * @license      GNU/GPLv2 http://www.gnu.org/licenses/
 * @link         http://www.nova-cms.com
 *
 */

if (!defined('FUNC_FILE')) die('Access is limited');

/**
* The function checks whether a given name, a name search engine
* 
* @param string $name
* @return bool
*/
function is_bot($name=""){
global $list_bots;
    if(!empty($name)) return isset($list_bots[$name]) ? true : false;
    else return isset($list_bots[$_SESSION['user']]) ? true : false;
}

/**
* Check IP addresses for validity
* 
* @param string $ip
* @return bool
*/
function validip($ip) {
global $reserved_ips;
    if (!empty($ip) AND $ip == long2ip(ip2long($ip))){
        foreach ($reserved_ips as $r) if ((ip2long($ip) >= $r[0]) AND (ip2long($ip) <= $r[1])) return false;
        return true;
    } else return false;
}

/**
* Checks support gzip compression
* 
* @return bool
*/
function check_can_gzip(){
    if(headers_sent() || connection_aborted() || !function_exists('ob_gzhandler') || ini_get('zlib.output_compression')) return 0;
    if(get_env('HTTP_ACCEPT_ENCODING')){
        $encoding=get_env('HTTP_ACCEPT_ENCODING');
        if(nc_strpos($encoding, 'x-gzip') !== false) return "x-gzip";
        if(nc_strpos($encoding, 'gzip') !== false) return "gzip";
    }
    return false;
}

/**
* Function to check whether the main page
* 
* @return bool
*/
function is_home(){
global $adminfile, $code2languages, $main;
    $uri=get_env('REQUEST_URI');
    if($uri=="/" OR $uri=="/index.php" OR $uri=="/index{$main->config['file_rewrite']}" OR $uri=="/".$adminfile OR !empty($code2languages[str_replace("/", "", $uri)])) return true; return false;
}

/**
* The function checks whether the user is an administrator or moderator
* 
* @return bool
*/
function is_support($id = 0){
    if(is_admin() OR is_moder($id)) return true; else return false;
}

/**
* The function checks whether the user is an administrator
* 
* @return bool
*/
function is_admin(){
    if(isset($_SESSION['admin']) AND is_user() AND !is_bot()) return true; else return false;
}

/**
* The function checks whether the user is moderator
* 
* @param int $forum_id
* @return bool
*/
function is_moder($id = 0){
global $userinfo, $main;
    if($id!=0){
        if(!isset($_SESSION['forum'][$main->user['user_name']][$id])){
            $result = $main->db->sql_query("SELECT * FROM ".ACC." WHERE uid='{$userinfo['uid']}' AND id='{$id}'");
            if($main->db->sql_numrows($result)>0) $_SESSION['forum'][$main->user['user_name']][$id] = true;
            else $_SESSION['forum'][$main->user['user_name']][$id] = false;
        }
        return $_SESSION['forum'][$main->user['user_name']][$id];
    }elseif(isset($_SESSION['admin']) AND $userinfo['user_level']=="1" AND is_user() AND !is_bot()) return true; else return false;
}

/**
* Function checks whether the visitor is an authorized user
* 
* @return bool
*/
function is_user(){
global $userconf;
    if(isset($_SESSION['user']) AND $_SESSION['user']!=$userconf['guest_name'] AND !is_bot()) return true; else return false;
}

/**
* The function checks whether the passed name, guest name
* 
* @param string $name
* @return bool
*/
function is_guest_name($name){
global $userconf;
    if($name==$userconf['guest_name']) return true; else return false;
}

/**
* The function checks whether the visitor is a guest
* 
* @return bool
*/
function is_guest(){
global $userconf;
    if(isset($_SESSION['user']) AND ($_SESSION['user']==$userconf['guest_name'] OR is_bot())) return true; else return false;
}

/**
* Function verification of Ajax. Whether the script as Ajax application
* 
* @return bool
*/
function is_ajax(){
    return ((isset($_SERVER['HTTP_X_REQUESTED_WITH']) AND $_SERVER['HTTP_X_REQUESTED_WITH']=='XMLHttpRequest') OR isset($_GET['ajax']) OR isset($_POST['ajax'])) ? true : false;
}

/**
* Function determines whether a string is an IP address
* 
* @param string $string
* @return bool
*/
function is_ip($string){
    if(preg_match('/^([0-9]|[0-9][0-9]|[01][0-9][0-9]|2[0-4][0-9]|25[0-5])(\.([0-9]|[0-9][0-9]|[01][0-9][0-9]|2[0-4][0-9]|25[0-5])){3}$/', $string)) return true;
    else return false;
}

/**
* The function checks whether the user is in the list submitted to the Panel
* 
* @param string $groups
* @return bool
*/
function check_user_group($groups){
global $main;
    if(empty($groups)) return true;
    else $arr = explode(',', $groups);
    $user_groups = array($main->user['user_group'])+(!empty($main->user['user_groups'])?explode(',', $main->user['user_groups']):array());    
    $array = array_intersect($arr, $user_groups);
    $_array = $array;
    foreach($_array as $key => $value) if(empty($array[$key])) unset($array[$key]);
    if(empty($array)) return false;
    else return true;
}

/**
* Function testing UTF8 encoding
* 
* @param string $string
* @return bool
*/
function is_utf8($string=""){
    return preg_match('%(?:
        [\xC2-\xDF][\x80-\xBF]                    # non-overlong 2-byte
        |\xE0[\xA0-\xBF][\x80-\xBF]               # excluding overlongs
        |[\xE1-\xEC\xEE\xEF][\x80-\xBF]{2}        # straight 3-byte
        |\xED[\x80-\x9F][\x80-\xBF]               # excluding surrogates
        |\xF0[\x90-\xBF][\x80-\xBF]{2}            # planes 1-3
        |[\xF1-\xF3][\x80-\xBF]{3}                # planes 4-15
        |\xF4[\x80-\x8F][\x80-\xBF]{2}            # plane 16
        )+%xs', $string);
}
?>